@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 import os
 import boto3
+import botocore
 import json
 import time
 import copy
@@ -31,7 +32,8 @@ from .vars import (
     SFN_TYPE,
     LAMBDA_TYPE,
     RUN_TASK_LAMBDA_NAME,
-    CHECK_TASK_LAMBDA_NAME
+    CHECK_TASK_LAMBDA_NAME,
+    UPDATE_COST_LAMBDA_NAME
 )
 from .utils import (
     _tibanna_settings,
@@ -39,6 +41,7 @@ from .utils import (
     does_key_exist,
     read_s3,
     upload,
+    put_object_s3,
     retrieve_all_keys,
     delete_keys
 )
@@ -46,9 +49,18 @@ from .ec2_utils import (
     UnicornInput,
     upload_workflow_to_s3
 )
+from .pricing_utils import (
+    get_cost,
+    get_cost_estimate,
+    update_cost_estimate_in_tsv,
+    update_cost_in_tsv,
+    get_cost_estimate_from_tsv
+)
+from .job import Job
 from .ami import AMI
 # from botocore.errorfactory import ExecutionAlreadyExists
 from .stepfunction import StepFunctionUnicorn
+from .stepfunction_cost_updater import StepFunctionCostUpdater
 from .awsem import AwsemRunJson, AwsemPostRunJson
 from .exceptions import (
     MetricRetrievalException
@@ -76,8 +88,11 @@ class API(object):
 
     @property
     def lambda_names(self):
-        return [mod for mod in dir(self.lambdas_module)
-                if isinstance(getattr(self.lambdas_module, mod), ModuleType)]
+        lambdas = [mod for mod in dir(self.lambdas_module)
+            if isinstance(getattr(self.lambdas_module, mod), ModuleType)]
+        # We remove update_cost_awsem, as this typically treated separately
+        lambdas.remove(self.update_cost_lambda)
+        return lambdas
 
     @property
     def tibanna_packages(self):
@@ -85,6 +100,7 @@ class API(object):
         return [tibanna]
 
     StepFunction = StepFunctionUnicorn
+    StepFunctionCU = StepFunctionCostUpdater
     default_stepfunction_name = TIBANNA_DEFAULT_STEP_FUNCTION_NAME
     default_env = ''
     sfn_type = SFN_TYPE
@@ -92,6 +108,7 @@ class API(object):
 
     run_task_lambda = RUN_TASK_LAMBDA_NAME
     check_task_lambda = CHECK_TASK_LAMBDA_NAME
+    update_cost_lambda = UPDATE_COST_LAMBDA_NAME
 
     @property
     def UNICORN_LAMBDAS(self):
@@ -196,8 +213,30 @@ class API(object):
             time.sleep(sleep)
         except Exception as e:
             raise(e)
+
+        # trigger the cost updater step function to run
+        try:
+            costupdater_input = {
+                "sfn_arn": data[_tibanna]['exec_arn'],
+                "log_bucket": data['config']['log_bucket'],
+                "job_id": data['jobid'],
+                "aws_region": AWS_REGION
+            }
+            costupdater_input = json.dumps(costupdater_input)
+            response = client.start_execution(
+                stateMachineArn=STEP_FUNCTION_ARN(sfn + "_costupdater"),
+                name=run_name,
+                input=costupdater_input,
+            )
+            time.sleep(sleep)
+        except Exception as e:
+            if 'StateMachineDoesNotExist' in str(e):
+                pass # Tibanna was probably deployed without the cost updater
+            else:
+                raise e
+
         # adding execution info to dynamoDB for fast search by awsem job id
-        self.add_to_dydb(jobid, run_name, sfn, data['config']['log_bucket'], verbose=verbose)
+        Job.add_to_dd(jobid, run_name, sfn, data['config']['log_bucket'], verbose=verbose)
         data[_tibanna]['response'] = response
         if verbose:
             # print some info
@@ -224,109 +263,24 @@ class API(object):
             run_infos.append(run_info)
         return run_infos
 
-    def add_to_dydb(self, awsem_job_id, execution_name, sfn, logbucket, verbose=True):
-        time_stamp = datetime.strftime(datetime.utcnow(), '%Y%m%d-%H:%M:%S-UTC')
-        dydb = boto3.client('dynamodb', region_name=AWS_REGION)
-        try:
-            # first check the table exists
-            res = dydb.describe_table(TableName=DYNAMODB_TABLE)
-        except Exception as e:
-            if verbose:
-                logger.info("Not adding to dynamo table: %s" % e)
-            return
-        try:
-            response = dydb.put_item(
-                TableName=DYNAMODB_TABLE,
-                Item={
-                    'Job Id': {
-                        'S': awsem_job_id
-                    },
-                    'Execution Name': {
-                        'S': execution_name
-                    },
-                    'Step Function': {
-                        'S': sfn
-                    },
-                    'Log Bucket': {
-                        'S': logbucket
-                    },
-                    'Time Stamp': {
-                        'S': time_stamp
-                    }
-                }
-            )
-            if verbose:
-                logger.info("Successfully put item to dynamoDB: " + str(response))
-        except Exception as e:
-            raise(e)
-
     def check_status(self, exec_arn=None, job_id=None):
-        '''checking status of an execution.
-        It works only if the execution info is still in the step function.'''
-        if not exec_arn and job_id:
-            ddinfo = self.info(job_id)
-            if not ddinfo:
-                raise Exception("Can't find exec_arn from the job_id")
-            exec_name = ddinfo.get('Execution Name', '')
-            sfn = ddinfo.get('Step Function', '')
-            exec_arn = EXECUTION_ARN(exec_name, sfn)
-            if not exec_arn:
-                raise Exception("Can't find exec_arn from the job_id")
-        sts = boto3.client('stepfunctions', region_name=AWS_REGION)
-        return sts.describe_execution(executionArn=exec_arn)['status']
+        """checking status of an execution.
+        """
+        return Job(exec_arn=exec_arn, job_id=job_id).check_status()
+
+    def check_costupdater_status(self, job_id=None):
+        '''checking status of the cost updater'''
+        return Job(job_id=job_id).check_costupdater_status()
 
     def check_output(self, exec_arn=None, job_id=None):
-        '''checking status of an execution first and if it's success, get output.
-        It works only if the execution info is still in the step function.'''
-        if not exec_arn and job_id:
-            ddinfo = self.info(job_id)
-            if not ddinfo:
-                raise Exception("Can't find exec_arn from the job_id")
-            exec_name = ddinfo.get('Execution Name', '')
-            sfn = ddinfo.get('Step Function', '')
-            exec_arn = EXECUTION_ARN(exec_name, sfn)
-            if not exec_arn:
-                raise Exception("Can't find exec_arn from the job_id")
-        sts = boto3.client('stepfunctions', region_name=AWS_REGION)
-        if self.check_status(exec_arn) == 'SUCCEEDED':
-            desc = sts.describe_execution(executionArn=exec_arn)
-            if 'output' in desc:
-                return json.loads(desc['output'])
-            else:
-                return None
-
-    def get_dd(self, job_id):
-        '''return raw content from dynamodb for a given job id'''
-        ddres = dict()
-        try:
-            dd = boto3.client('dynamodb')
-            ddres = dd.query(TableName=DYNAMODB_TABLE,
-                             KeyConditions={'Job Id': {'AttributeValueList': [{'S': job_id}],
-                                                       'ComparisonOperator': 'EQ'}})
-            return ddres
-        except Exception as e:
-            logger.warning("DynamoDB entry not found: %s" % e)
-            return None
+        """checking status of an execution first and if it's success, get output.
+        It works only if the execution info is still in the step function.
+        """
+        return Job(exec_arn=exec_arn, job_id=job_id).check_output()
 
     def info(self, job_id):
         '''returns content from dynamodb for a given job id in a dictionary form'''
-        ddres = self.get_dd(job_id)
-        return self.get_info_from_dd(ddres)
-
-    def get_info_from_dd(self, ddres):
-        '''converts raw content from dynamodb to a dictionary form'''
-        if not ddres:
-            return None
-        if 'Items' in ddres:
-            try:
-                dditem = ddres['Items'][0]
-                return dd_utils.item2dict(dditem)
-            except Exception as e:
-                logger.warning("DynamoDB fields not found: %s" % e)
-                return None
-        else:
-            logger.warning("DynamoDB Items field not found:")
-            return None
+        return Job.info(job_id)
 
     def kill(self, exec_arn=None, job_id=None, sfn=None):
         sf = boto3.client('stepfunctions')
@@ -677,7 +631,10 @@ class API(object):
             if shutdown_min:
                 override_config['shutdown_min'] = shutdown_min
             if ebs_size:
-                override_config['ebs_size'] = int(ebs_size)
+                if ebs_size.endswith('x'):
+                    override_config['ebs_size'] = ebs_size
+                else:
+                    override_config['ebs_size'] = int(ebs_size)
             if overwrite_input_extra:
                 override_config['overwrite_input_extra'] = overwrite_input_extra
             if key_name:
@@ -765,7 +722,7 @@ class API(object):
                 extra_config['Environment']['Variables']['AWS_S3_ROLE_NAME'] = 'S3_access'  # 4dn-dcic default(temp)
         # add role
         logger.info('name=%s' % name)
-        if name in [self.run_task_lambda, self.check_task_lambda]:
+        if name in [self.run_task_lambda, self.check_task_lambda, self.update_cost_lambda]:
             role_arn_prefix = 'arn:aws:iam::' + AWS_ACCOUNT_NUMBER + ':role/'
             if usergroup:
                 role_arn = role_arn_prefix + tibanna_iam.role_name(name)
@@ -773,6 +730,7 @@ class API(object):
                 role_arn = role_arn_prefix + 'lambda_full_s3'  # 4dn-dcic default(temp)
             logger.info("role_arn=" + role_arn)
             extra_config['Role'] = role_arn
+
         if usergroup and suffix:
             function_name_suffix = usergroup + '_' + suffix
         elif suffix:
@@ -781,6 +739,7 @@ class API(object):
             function_name_suffix = usergroup
         else:
             function_name_suffix = ''
+
         # first delete the existing function to avoid the weird AWS KMS lambda error
         function_name_prefix = getattr(self.lambdas_module, name).config.get('function_name')
         if function_name_suffix:
@@ -795,6 +754,7 @@ class API(object):
             except Exception as e:
                 if 'Function not found' in str(e):
                     pass
+
         aws_lambda.deploy_function(lambda_fxn_module,
                                    function_name_suffix=function_name_suffix,
                                    package_objects=self.tibanna_packages,
@@ -843,7 +803,8 @@ class API(object):
         return tibanna_iam.user_group_name
 
     def deploy_tibanna(self, suffix=None, usergroup='', setup=False,
-                       buckets='', setenv=False, do_not_delete_public_access_block=False):
+                       buckets='', setenv=False, do_not_delete_public_access_block=False,
+                       deploy_costupdater=False):
         """deploy tibanna unicorn or pony to AWS cloud (pony is for 4DN-DCIC only)"""
         if setup:
             if usergroup:
@@ -855,21 +816,32 @@ class API(object):
         # this function will remove existing step function on a conflict
         step_function_name = self.create_stepfunction(suffix, usergroup=usergroup)
         logger.info("creating a new step function... %s" % step_function_name)
+
+        if(deploy_costupdater):
+            step_function_cu_name = self.create_stepfunction(suffix, usergroup=usergroup, costupdater=True)
+            logger.info("creating a new step function... %s" % step_function_cu_name)
+
         if setenv:
             os.environ['TIBANNA_DEFAULT_STEP_FUNCTION_NAME'] = step_function_name
             with open(os.getenv('HOME') + "/.bashrc", "a") as outfile:  # 'a' stands for "append"
                 outfile.write("\nexport TIBANNA_DEFAULT_STEP_FUNCTION_NAME=%s\n" % step_function_name)
+
         logger.info("deploying lambdas...")
         self.deploy_core('all', suffix=suffix, usergroup=usergroup)
+
+        if(deploy_costupdater):
+            self.deploy_lambda(self.update_cost_lambda, suffix=suffix, usergroup=usergroup)
+
         dd_utils.create_dynamo_table(DYNAMODB_TABLE, DYNAMODB_KEYNAME)
         return step_function_name
 
     def deploy_unicorn(self, suffix=None, no_setup=False, buckets='',
-                       no_setenv=False, usergroup='', do_not_delete_public_access_block=False):
+                       no_setenv=False, usergroup='', do_not_delete_public_access_block=False, deploy_costupdater = False):
         """deploy tibanna unicorn to AWS cloud"""
         self.deploy_tibanna(suffix=suffix, usergroup=usergroup, setup=not no_setup,
                             buckets=buckets, setenv=not no_setenv,
-                            do_not_delete_public_access_block=do_not_delete_public_access_block)
+                            do_not_delete_public_access_block=do_not_delete_public_access_block,
+                            deploy_costupdater=deploy_costupdater)
 
     def add_user(self, user, usergroup):
         """add a user to a tibanna group"""
@@ -906,12 +878,16 @@ class API(object):
     def create_stepfunction(self, dev_suffix=None,
                             region_name=AWS_REGION,
                             aws_acc=AWS_ACCOUNT_NUMBER,
-                            usergroup=None):
+                            usergroup=None,
+                            costupdater=False):
         if not aws_acc or not region_name:
             logger.info("Please set and export environment variable AWS_ACCOUNT_NUMBER and AWS_REGION!")
             exit(1)
         # create a step function definition object
-        sfndef = self.StepFunction(dev_suffix, region_name, aws_acc, usergroup)
+        if(costupdater):
+            sfndef = self.StepFunctionCU(dev_suffix, region_name, aws_acc, usergroup)
+        else:
+            sfndef = self.StepFunction(dev_suffix, region_name, aws_acc, usergroup)
         # if this encouters an existing step function with the same name, delete
         sfn = boto3.client('stepfunctions', region_name=region_name)
         retries = 12  # wait 10 seconds between retries for total of 120s
@@ -1035,7 +1011,12 @@ class API(object):
             self.TibannaResource.update_html(log_bucket, job_id + '.metrics/')
         else:
             try:
-                M = self.TibannaResource(instance_id, filesystem, starttime, endtime)
+                # We don't want to recompute the cost estimate - we take it from the tsv
+                cost_estimate, cost_estimate_type = get_cost_estimate_from_tsv(log_bucket, job_id)
+                if(cost_estimate == 0.0): # Cost estimate is not yet in tsv -> compute it
+                    cost_estimate, cost_estimate_type = self.cost_estimate(job_id=job_id)
+
+                M = self.TibannaResource(instance_id, filesystem, starttime, endtime, cost_estimate = cost_estimate, cost_estimate_type=cost_estimate_type)
                 top_content = self.log(job_id=job_id, top=True)
                 M.plot_metrics(instance_type, directory, top_content=top_content)
             except Exception as e:
@@ -1050,6 +1031,31 @@ class API(object):
         if open_browser:
             webbrowser.open(METRICS_URL(log_bucket, job_id))
 
+    def cost_estimate(self, job_id, update_tsv=False):
+        postrunjsonstr = self.log(job_id=job_id, postrunjson=True)
+        if not postrunjsonstr:
+            logger.info("Cost estimation error: postrunjson not found")
+            return 0.0, "NA"
+        postrunjsonobj = json.loads(postrunjsonstr)
+        postrunjson = AwsemPostRunJson(**postrunjsonobj)
+        log_bucket = postrunjson.config.log_bucket
+
+        # We return the real cost, if it is availble, but don't automatically update the Cost row in the tsv
+        precise_cost = self.cost(job_id, update_tsv=False)
+        if(precise_cost and precise_cost > 0.0):
+            if update_tsv:
+                update_cost_estimate_in_tsv(log_bucket, job_id, precise_cost, cost_estimate_type="actual cost")
+            return precise_cost, "actual cost"
+
+        # awsf_image was added in 1.0.0. We use that to get the correct ebs root type
+        ebs_root_type = 'gp3' if 'awsf_image' in postrunjsonobj['config'] else 'gp2'
+
+        cost_estimate, cost_estimate_type = get_cost_estimate(postrunjson, ebs_root_type)
+
+        if update_tsv:
+            update_cost_estimate_in_tsv(log_bucket, job_id, cost_estimate, cost_estimate_type)
+        return cost_estimate, cost_estimate_type
+
     def cost(self, job_id, sfn=None, update_tsv=False):
         if not sfn:
             sfn = self.default_stepfunction_name
@@ -1057,38 +1063,16 @@ class API(object):
         if not postrunjsonstr:
             return None
         postrunjson = AwsemPostRunJson(**json.loads(postrunjsonstr))
-        job = postrunjson.Job
 
-        def reformat_time(t, delta):
-            d = datetime.strptime(t, '%Y%m%d-%H:%M:%S-UTC') + timedelta(days=delta)
-            return d.strftime("%Y-%m-%d")
+        cost = get_cost(postrunjson, job_id)
 
-        start_time = reformat_time(job.start_time, -1)  # give more room
-        end_time = reformat_time(job.end_time, 1)  # give more room
-        billing_args = {'Filter': {'Tags': {'Key': 'Name', 'Values': ['awsem-' + job_id]}},
-                        'Granularity': 'DAILY',
-                        'TimePeriod': {'Start': start_time,
-                                       'End': end_time},
-                        'Metrics': ['BlendedCost']}
-        billingres = boto3.client('ce').get_cost_and_usage(**billing_args)
-        cost = sum([float(_['Total']['BlendedCost']['Amount']) for _ in billingres['ResultsByTime']])
         if update_tsv:
             log_bucket = postrunjson.config.log_bucket
-            # reading from metrics_report.tsv
-            does_key_exist(log_bucket, job_id + '.metrics/metrics_report.tsv')
-            read_file = read_s3(log_bucket, os.path.join(job_id + '.metrics/', 'metrics_report.tsv'))
-            if 'Cost' not in read_file:
-                write_file = read_file + 'Cost\t' + str(cost) + '\n'
-                # writing
-                with open('metrics_report.tsv', 'w') as fo:
-                    fo.write(write_file)
-                # upload new metrics_report.tsv
-                upload('metrics_report.tsv', log_bucket, job_id + '.metrics/')
-                os.remove('metrics_report.tsv')
-            else:
-                logger.info("cost already in the tsv file. not updating")
+            update_cost_in_tsv(log_bucket, job_id, cost)
+
         return cost
 
+
     def does_dynamo_table_exist(self, tablename):
         try:
             res = boto3.client('dynamodb').describe_table(
@@ -1158,7 +1142,7 @@ class API(object):
             lambda_suffix = '_' + user_group_name + '_' + suffix
         else:
             lambda_suffix = '_' + user_group_name
-        # delete step function
+        # delete step functions
         sfn = 'tibanna_' + self.sfn_type + lambda_suffix
         if verbose:
             logger.info("deleting step function %s" % sfn)
@@ -1166,6 +1150,16 @@ class API(object):
             boto3.client('stepfunctions').delete_state_machine(stateMachineArn=STEP_FUNCTION_ARN(sfn))
         except Exception as e:
             handle_error("Failed to cleanup step function: %s" % str(e))
+
+        sfn_costupdater = 'tibanna_' + self.sfn_type + lambda_suffix + '_costupdater'
+        if verbose:
+            logger.info("deleting step function %s" % sfn_costupdater)
+        try:
+            # This does not produce an exception, if the step function does not exist
+            boto3.client('stepfunctions').delete_state_machine(stateMachineArn=STEP_FUNCTION_ARN(sfn_costupdater))
+        except Exception as e:
+            handle_error("Failed to cleanup step function: %s" % str(e))
+
         # delete lambdas
         lambda_client = boto3.client('lambda')
         for lmb in self.lambda_names:
@@ -1175,6 +1169,14 @@ class API(object):
                 lambda_client.delete_function(FunctionName=lmb + lambda_suffix)
             except Exception as e:
                 handle_error("Failed to cleanup lambda: %s" % str(e))
+
+        if verbose:
+            logger.info("deleting lambda functions %s" % self.update_cost_lambda + lambda_suffix)
+        try:
+            lambda_client.delete_function(FunctionName=self.update_cost_lambda  + lambda_suffix)
+        except Exception as e:
+            handle_error("Failed to cleanup lambda: %s" % str(e))
+
         # delete IAM policies, roles and groups
         if not do_not_remove_iam_group:
             if verbose:
@@ -1221,4 +1223,3 @@ class API(object):
                 args.update({'base_region': source_image_region})
 
         return AMI(**args).create_ami_for_tibanna(make_public=make_public)
-
