@@ -7,8 +7,10 @@ from .vars import (
     AWS_ACCOUNT_NUMBER,
     AWS_REGION,
     LAMBDA_TYPE,
+    SFN_TYPE,
     RUN_TASK_LAMBDA_NAME,
-    CHECK_TASK_LAMBDA_NAME
+    CHECK_TASK_LAMBDA_NAME,
+    UPDATE_COST_LAMBDA_NAME
 )
 
 
@@ -20,8 +22,10 @@ class IAM(object):
     account_id = AWS_ACCOUNT_NUMBER
     region = AWS_REGION
     lambda_type = LAMBDA_TYPE  # lambda_type : '' for unicorn, 'pony' for pony, 'zebra' for zebra
+    sfn_type = SFN_TYPE  # sfn type : 'unicorn' for unicorn, 'pony' for pony, 'zebra' for zebra
     run_task_lambda_name = RUN_TASK_LAMBDA_NAME
     check_task_lambda_name = CHECK_TASK_LAMBDA_NAME
+    update_cost_lambda_name = UPDATE_COST_LAMBDA_NAME
 
     def __init__(self, user_group_tag, bucket_names='', no_randomize=True):
         """policy prefix for user group
@@ -33,7 +37,7 @@ class IAM(object):
           prefix : tibanna_ / tibanna_pony_
         """
         # lambda names
-        self.lambda_names = [self.run_task_lambda_name, self.check_task_lambda_name]
+        self.lambda_names = [self.run_task_lambda_name, self.check_task_lambda_name, self.update_cost_lambda_name]
         if self.lambda_type:
             self.prefix = 'tibanna_' + self.lambda_type + '_'
         else:
@@ -48,6 +52,7 @@ class IAM(object):
             random_tag = str(int(random.random() * 10000))
             self.user_group_name = self.user_group_tag + '_' + random_tag
         self.tibanna_policy_prefix = self.prefix + self.user_group_name
+        self.tibanna_sfn_name = self.prefix + self.sfn_type + '_' + self.user_group_name
 
         # bucket names
         self.bucket_names = bucket_names
@@ -63,7 +68,8 @@ class IAM(object):
     @property
     def policy_types(self):
         return ['bucket', 'termination', 'list', 'cloudwatch', 'passrole', 'lambdainvoke',
-                'desc_stepfunction', 'cloudwatch_metric', 'cw_dashboard', 'dynamodb', 'ec2_desc']
+                'cloudwatch_metric', 'cw_dashboard', 'dynamodb', 'ec2_desc',
+                'executions', 'pricing']
 
     def policy_arn(self, policy_type):
         return 'arn:aws:iam::' + self.account_id + ':policy/' + self.policy_name(policy_type)
@@ -75,11 +81,12 @@ class IAM(object):
                     'cloudwatch': 'cloudwatchlogs',
                     'passrole': 'iam_passrole_s3',
                     'lambdainvoke': 'lambdainvoke',
-                    'desc_stepfunction': 'desc_sts',
                     'cloudwatch_metric': 'cw_metric',
                     'cw_dashboard': 'cw_dashboard',
                     'dynamodb': 'dynamodb',
-                    'ec2_desc': 'ec2_desc'}
+                    'ec2_desc': 'ec2_desc',
+                    'pricing': 'pricing',
+                    'executions': 'executions'}
         if policy_type not in suffices:
             raise Exception("policy %s must be one of %s." % (policy_type, str(self.policy_types)))
         return suffices[policy_type]
@@ -94,11 +101,12 @@ class IAM(object):
                        'cloudwatch': self.policy_cloudwatchlogs,
                        'passrole': self.policy_iam_passrole_s3,
                        'lambdainvoke': self.policy_lambdainvoke,
-                       'desc_stepfunction': self.policy_desc_stepfunction,
                        'cloudwatch_metric': self.policy_cloudwatch_metric,
                        'cw_dashboard': self.policy_cw_dashboard,
                        'dynamodb': self.policy_dynamodb,
-                       'ec2_desc': self.policy_ec2_desc_policy}
+                       'ec2_desc': self.policy_ec2_desc_policy,
+                       'pricing': self.policy_pricing,
+                       'executions': self.policy_executions}
         if policy_type not in definitions:
             raise Exception("policy %s must be one of %s." % (policy_type, str(self.policy_types)))
         return definitions[policy_type]
@@ -128,16 +136,19 @@ class IAM(object):
 
     def policy_arn_list_for_role(self, role_type):
         run_task_custom_policy_types = ['list', 'cloudwatch', 'passrole', 'bucket', 'dynamodb',
-                                        'desc_stepfunction', 'cw_dashboard']
+                                        'executions', 'cw_dashboard']
         check_task_custom_policy_types = ['cloudwatch_metric', 'cloudwatch', 'bucket', 'ec2_desc',
-                                          'termination', 'dynamodb']
+                                          'termination', 'dynamodb', 'pricing']
+        update_cost_custom_policy_types = ['bucket', 'executions', 'dynamodb', 'pricing']
         arnlist = {'ec2': [self.policy_arn(_) for _ in ['bucket', 'cloudwatch_metric']] +
                           ['arn:aws:iam::aws:policy/AmazonEC2ContainerRegistryReadOnly'],
                    # 'stepfunction': [self.policy_arn(_) for _ in ['lambdainvoke']],
                    'stepfunction': ['arn:aws:iam::aws:policy/service-role/AWSLambdaRole'],
                    self.run_task_lambda_name: [self.policy_arn(_) for _ in run_task_custom_policy_types] +
                                               ['arn:aws:iam::aws:policy/AmazonEC2FullAccess'],
-                   self.check_task_lambda_name: [self.policy_arn(_) for _ in check_task_custom_policy_types]}
+                   self.check_task_lambda_name: [self.policy_arn(_) for _ in check_task_custom_policy_types],
+                   self.update_cost_lambda_name: [self.policy_arn(_) for _ in update_cost_custom_policy_types]}
+                   
         if role_type not in arnlist:
             raise Exception("role_type %s must be one of %s." % (role_type, str(self.role_types)))
         return arnlist[role_type]
@@ -172,7 +183,8 @@ class IAM(object):
                         "s3:PutObject",
                         "s3:GetObject",
                         "s3:DeleteObject",
-                        "s3:PutObjectAcl"
+                        "s3:PutObjectAcl",
+                        "s3:PutObjectTagging"
                     ],
                     "Resource": resource_list_objects
                 }
@@ -268,18 +280,32 @@ class IAM(object):
         return policy
 
     @property
-    def policy_desc_stepfunction(self):
-        execution_arn_prefix = 'arn:aws:states:' + self.region + ':' + self.account_id + ':execution:'
-        resource = execution_arn_prefix + self.tibanna_policy_prefix + ':*'
+    def policy_executions(self):
+        execution_arn_prefix = 'arn:aws:states:' + self.region + ':' + self.account_id + ':stateMachine:'
+        sfn_arn_prefix = 'arn:aws:states:' + self.region + ':' + self.account_id + ':execution:'
+        resources = [execution_arn_prefix + self.tibanna_sfn_name,
+                    sfn_arn_prefix + self.tibanna_sfn_name + ':*']
         policy = {
             "Version": "2012-10-17",
             "Statement": [
                 {
                     "Effect": "Allow",
                     "Action": [
-                        "states:DescribeExecution"
+                        "states:StartExecution",
+                        "states:StopExecution",
+                        "states:ListExecutions",
+                        "states:DescribeExecution",
+                        "states:GetExecutionHistory",
+                        "states:DescribeStateMachineForExecution",
+                        "states:DescribeStateMachine"
                     ],
-                    "Resource": resource
+                    "Resource": resources
+                },
+                {
+                    "Sid": "VisualEditor1",
+                    "Effect": "Allow",
+                    "Action": "states:ListStateMachines",
+                    "Resource": "*"
                 }
             ]
         }
@@ -345,7 +371,27 @@ class IAM(object):
                     "Effect": "Allow",
                     "Action": [
                         "ec2:DescribeInstances",
-                        "ec2:DescribeInstanceStatus"
+                        "ec2:DescribeInstanceStatus",
+                        "ec2:DescribeSpotPriceHistory"
+                    ],
+                    "Resource": "*"
+                }
+            ]
+        }
+        return policy
+    
+    @property
+    def policy_pricing(self):
+        policy = {
+            "Version": "2012-10-17",
+            "Statement": [
+                {
+                    "Effect": "Allow",
+                    "Action": [
+                        "pricing:DescribeServices",
+                        "pricing:GetAttributeValues",
+                        "pricing:GetProducts",
+                        "ce:GetCostAndUsage"
                     ],
                     "Resource": "*"
                 }
@@ -421,16 +467,6 @@ class IAM(object):
                 # deleting a group would require users to be detached from the group.
                 self.detach_policies_from_group()
         group = self.iam.Group(self.iam_group_name)
-        response = group.attach_policy(
-            PolicyArn='arn:aws:iam::aws:policy/AWSStepFunctionsFullAccess'
-        )
-        if verbose:
-            logger.debug("response from IAM attach_policy :" + str(response))
-        response = group.attach_policy(
-            PolicyArn='arn:aws:iam::aws:policy/AWSStepFunctionsConsoleFullAccess'
-        )
-        if verbose:
-            logger.debug("response from IAM attach_policy :" + str(response))
         response = group.attach_policy(
             PolicyArn='arn:aws:iam::aws:policy/service-role/AWSLambdaBasicExecutionRole'
         )
@@ -441,7 +477,7 @@ class IAM(object):
         )
         if verbose:
             logger.debug("response from IAM attach_policy :" + str(response))
-        custom_policy_types = ['bucket', 'ec2_desc', 'cloudwatch_metric', 'dynamodb', 'termination']
+        custom_policy_types = ['bucket', 'ec2_desc', 'cloudwatch_metric', 'dynamodb', 'termination', 'pricing', 'executions']
         for pn in [self.policy_name(pt) for pt in custom_policy_types]:
             response = group.attach_policy(
                 PolicyArn='arn:aws:iam::' + self.account_id + ':policy/' + pn
